@@ -361,7 +361,7 @@ impl PartialEq<FixedSizeBinaryArray> for Value {
     }
 }
 
-impl JsonEqual for DecimalArray {
+impl JsonEqual for Decimal128Array {
     fn equals_json(&self, json: &[&Value]) -> bool {
         if self.len() != json.len() {
             return false;
@@ -394,7 +394,7 @@ impl JsonEqual for Decimal256Array {
     }
 }
 
-impl PartialEq<Value> for DecimalArray {
+impl PartialEq<Value> for Decimal128Array {
     fn eq(&self, json: &Value) -> bool {
         match json {
             Value::Array(json_array) => self.equals_json_values(json_array),
@@ -403,8 +403,8 @@ impl PartialEq<Value> for DecimalArray {
     }
 }
 
-impl PartialEq<DecimalArray> for Value {
-    fn eq(&self, arrow: &DecimalArray) -> bool {
+impl PartialEq<Decimal128Array> for Value {
+    fn eq(&self, arrow: &Decimal128Array) -> bool {
         match self {
             Value::Array(json_array) => arrow.equals_json_values(json_array),
             _ => false,
@@ -468,10 +468,10 @@ mod tests {
     ) -> Result<ListArray> {
         for d in data.as_ref() {
             if let Some(v) = d {
-                builder.values().append_slice(v.as_ref())?;
-                builder.append(true)?
+                builder.values().append_slice(v.as_ref());
+                builder.append(true);
             } else {
-                builder.append(false)?
+                builder.append(false);
             }
         }
         Ok(builder.finish())
@@ -484,13 +484,13 @@ mod tests {
     ) -> Result<FixedSizeListArray> {
         for d in data.as_ref() {
             if let Some(v) = d {
-                builder.values().append_slice(v.as_ref())?;
-                builder.append(true)?
+                builder.values().append_slice(v.as_ref());
+                builder.append(true);
             } else {
                 for _ in 0..builder.value_length() {
-                    builder.values().append_null()?;
+                    builder.values().append_null();
                 }
-                builder.append(false)?
+                builder.append(false);
             }
         }
         Ok(builder.finish())
@@ -766,12 +766,12 @@ mod tests {
     fn test_binary_json_equal() {
         // Test the equal case
         let mut builder = BinaryBuilder::new(6);
-        builder.append_value(b"hello").unwrap();
-        builder.append_null().unwrap();
-        builder.append_null().unwrap();
-        builder.append_value(b"world").unwrap();
-        builder.append_null().unwrap();
-        builder.append_null().unwrap();
+        builder.append_value(b"hello");
+        builder.append_null();
+        builder.append_null();
+        builder.append_value(b"world");
+        builder.append_null();
+        builder.append_null();
         let arrow_array = builder.finish();
         let json_array: Value = serde_json::from_str(
             r#"
@@ -866,7 +866,7 @@ mod tests {
         // Test the equal case
         let mut builder = FixedSizeBinaryBuilder::new(15, 5);
         builder.append_value(b"hello").unwrap();
-        builder.append_null().unwrap();
+        builder.append_null();
         builder.append_value(b"world").unwrap();
         let arrow_array: FixedSizeBinaryArray = builder.finish();
         let json_array: Value = serde_json::from_str(
@@ -884,7 +884,7 @@ mod tests {
 
         // Test unequal case
         builder.append_value(b"hello").unwrap();
-        builder.append_null().unwrap();
+        builder.append_null();
         builder.append_value(b"world").unwrap();
         let arrow_array: FixedSizeBinaryArray = builder.finish();
         let json_array: Value = serde_json::from_str(
@@ -947,7 +947,7 @@ mod tests {
         // Test the equal case
         let arrow_array = [Some(1_000), None, Some(-250)]
             .iter()
-            .collect::<DecimalArray>()
+            .collect::<Decimal128Array>()
             .with_precision_and_scale(23, 6)
             .unwrap();
         let json_array: Value = serde_json::from_str(
@@ -966,7 +966,7 @@ mod tests {
         // Test unequal case
         let arrow_array = [Some(1_000), None, Some(55)]
             .iter()
-            .collect::<DecimalArray>()
+            .collect::<Decimal128Array>()
             .with_precision_and_scale(23, 6)
             .unwrap();
         let json_array: Value = serde_json::from_str(
