@@ -61,8 +61,8 @@ pub fn regexp_match<OffsetSize: OffsetSizeTrait>(
                 // Required for Postgres compatibility:
                 // SELECT regexp_match('foobarbequebaz', ''); = {""}
                 (Some(_), Some(pattern)) if pattern == *"" => {
-                    list_builder.values().append_value("")?;
-                    list_builder.append(true)?;
+                    list_builder.values().append_value("");
+                    list_builder.append(true);
                 }
                 (Some(value), Some(pattern)) => {
                     let existing_pattern = patterns.get(&pattern);
@@ -82,14 +82,14 @@ pub fn regexp_match<OffsetSize: OffsetSizeTrait>(
                     match re.captures(value) {
                         Some(caps) => {
                             for m in caps.iter().skip(1).flatten() {
-                                list_builder.values().append_value(m.as_str())?;
+                                list_builder.values().append_value(m.as_str());
                             }
-                            list_builder.append(true)?
+                            list_builder.append(true);
                         }
-                        None => list_builder.append(false)?,
+                        None => list_builder.append(false),
                     }
                 }
-                _ => list_builder.append(false)?,
+                _ => list_builder.append(false),
             }
             Ok(())
         })
@@ -103,7 +103,7 @@ mod tests {
     use crate::array::{ListArray, StringArray};
 
     #[test]
-    fn match_single_group() -> Result<()> {
+    fn match_single_group() {
         let values = vec![
             Some("abc-005-def"),
             Some("X-7-5"),
@@ -117,41 +117,39 @@ mod tests {
         pattern_values.push(r"(bar)(bequ1e)");
         pattern_values.push("");
         let pattern = StringArray::from(pattern_values);
-        let actual = regexp_match(&array, &pattern, None)?;
+        let actual = regexp_match(&array, &pattern, None).unwrap();
         let elem_builder: GenericStringBuilder<i32> = GenericStringBuilder::new(0);
         let mut expected_builder = ListBuilder::new(elem_builder);
-        expected_builder.values().append_value("005")?;
-        expected_builder.append(true)?;
-        expected_builder.values().append_value("7")?;
-        expected_builder.append(true)?;
-        expected_builder.append(false)?;
-        expected_builder.append(false)?;
-        expected_builder.append(false)?;
-        expected_builder.values().append_value("")?;
-        expected_builder.append(true)?;
+        expected_builder.values().append_value("005");
+        expected_builder.append(true);
+        expected_builder.values().append_value("7");
+        expected_builder.append(true);
+        expected_builder.append(false);
+        expected_builder.append(false);
+        expected_builder.append(false);
+        expected_builder.values().append_value("");
+        expected_builder.append(true);
         let expected = expected_builder.finish();
         let result = actual.as_any().downcast_ref::<ListArray>().unwrap();
         assert_eq!(&expected, result);
-        Ok(())
     }
 
     #[test]
-    fn match_single_group_with_flags() -> Result<()> {
+    fn match_single_group_with_flags() {
         let values = vec![Some("abc-005-def"), Some("X-7-5"), Some("X545"), None];
         let array = StringArray::from(values);
         let pattern = StringArray::from(vec![r"x.*-(\d*)-.*"; 4]);
         let flags = StringArray::from(vec!["i"; 4]);
-        let actual = regexp_match(&array, &pattern, Some(&flags))?;
+        let actual = regexp_match(&array, &pattern, Some(&flags)).unwrap();
         let elem_builder: GenericStringBuilder<i32> = GenericStringBuilder::new(0);
         let mut expected_builder = ListBuilder::new(elem_builder);
-        expected_builder.append(false)?;
-        expected_builder.values().append_value("7")?;
-        expected_builder.append(true)?;
-        expected_builder.append(false)?;
-        expected_builder.append(false)?;
+        expected_builder.append(false);
+        expected_builder.values().append_value("7");
+        expected_builder.append(true);
+        expected_builder.append(false);
+        expected_builder.append(false);
         let expected = expected_builder.finish();
         let result = actual.as_any().downcast_ref::<ListArray>().unwrap();
         assert_eq!(&expected, result);
-        Ok(())
     }
 }
