@@ -160,16 +160,14 @@ fn build_primitive_reader(
     ));
 
     let page_iterator = row_groups.column_chunks(col_idx)?;
-    let null_mask_only = field.def_level == 1 && field.nullable;
     let arrow_type = Some(field.arrow_type.clone());
 
     match physical_type {
         PhysicalType::BOOLEAN => Ok(Box::new(
-            PrimitiveArrayReader::<BoolType>::new_with_options(
+            PrimitiveArrayReader::<BoolType>::new(
                 page_iterator,
                 column_desc,
                 arrow_type,
-                null_mask_only,
             )?,
         )),
         PhysicalType::INT32 => {
@@ -180,21 +178,19 @@ fn build_primitive_reader(
                 )?))
             } else {
                 Ok(Box::new(
-                    PrimitiveArrayReader::<Int32Type>::new_with_options(
+                    PrimitiveArrayReader::<Int32Type>::new(
                         page_iterator,
                         column_desc,
                         arrow_type,
-                        null_mask_only,
                     )?,
                 ))
             }
         }
         PhysicalType::INT64 => Ok(Box::new(
-            PrimitiveArrayReader::<Int64Type>::new_with_options(
+            PrimitiveArrayReader::<Int64Type>::new(
                 page_iterator,
                 column_desc,
                 arrow_type,
-                null_mask_only,
             )?,
         )),
         PhysicalType::INT96 => {
@@ -218,19 +214,17 @@ fn build_primitive_reader(
             )?))
         }
         PhysicalType::FLOAT => Ok(Box::new(
-            PrimitiveArrayReader::<FloatType>::new_with_options(
+            PrimitiveArrayReader::<FloatType>::new(
                 page_iterator,
                 column_desc,
                 arrow_type,
-                null_mask_only,
             )?,
         )),
         PhysicalType::DOUBLE => Ok(Box::new(
-            PrimitiveArrayReader::<DoubleType>::new_with_options(
+            PrimitiveArrayReader::<DoubleType>::new(
                 page_iterator,
                 column_desc,
                 arrow_type,
-                null_mask_only,
             )?,
         )),
         PhysicalType::BYTE_ARRAY => match arrow_type {
@@ -238,13 +232,11 @@ fn build_primitive_reader(
                 page_iterator,
                 column_desc,
                 arrow_type,
-                null_mask_only,
             ),
             _ => make_byte_array_reader(
                 page_iterator,
                 column_desc,
                 arrow_type,
-                null_mask_only,
             ),
         },
         PhysicalType::FIXED_LEN_BYTE_ARRAY => match field.arrow_type {
