@@ -25,7 +25,7 @@ use crate::data_type::DataType;
 use crate::errors::{ParquetError, Result};
 use crate::schema::types::ColumnDescPtr;
 use arrow::array::{
-    ArrayDataBuilder, ArrayRef, BooleanArray, BooleanBufferBuilder, DecimalArray,
+    ArrayDataBuilder, ArrayRef, BooleanArray, BooleanBufferBuilder, Decimal128Array,
     Float32Array, Float64Array, Int32Array, Int64Array,
 };
 use arrow::buffer::Buffer;
@@ -58,17 +58,6 @@ where
         pages: Box<dyn PageIterator>,
         column_desc: ColumnDescPtr,
         arrow_type: Option<ArrowType>,
-    ) -> Result<Self> {
-        Self::new_with_options(pages, column_desc, arrow_type, false)
-    }
-
-    /// Construct primitive array reader with ability to only compute null mask and not
-    /// buffer level data
-    pub fn new_with_options(
-        pages: Box<dyn PageIterator>,
-        column_desc: ColumnDescPtr,
-        arrow_type: Option<ArrowType>,
-        null_mask_only: bool,
     ) -> Result<Self> {
         // Check if Arrow type is specified, else create it from Parquet type
         let data_type = match arrow_type {
@@ -78,8 +67,7 @@ where
                 .clone(),
         };
 
-        let record_reader =
-            RecordReader::<T>::new_with_options(column_desc.clone(), null_mask_only);
+        let record_reader = RecordReader::<T>::new(column_desc.clone());
 
         Ok(Self {
             data_type,
@@ -203,7 +191,7 @@ where
                         .unwrap()
                         .iter()
                         .map(|v| v.map(|v| v.into()))
-                        .collect::<DecimalArray>(),
+                        .collect::<Decimal128Array>(),
 
                     ArrowType::Int64 => array
                         .as_any()
@@ -211,7 +199,7 @@ where
                         .unwrap()
                         .iter()
                         .map(|v| v.map(|v| v.into()))
-                        .collect::<DecimalArray>(),
+                        .collect::<Decimal128Array>(),
                     _ => {
                         return Err(arrow_err!(
                             "Cannot convert {:?} to decimal",
